#!/usr/bin/env python
# coding=utf-8
from __future__ import print_function

import os
import subprocess
import time
import uuid

from .utils import (expand, get_repo_archive_name, iter_repositories,
                    parse_config, sha12, stdout_write)


def create_create_command(repository, prefix, setup, stats=False, verbosity=0):
    if 'paths' not in setup or not setup['paths']:
        return

    archive_name = '{prefix}-{date}-{uuid}'.format(
        prefix=prefix,
        date=time.strftime('%Y.%m.%dT%H:%M:%S'),
        uuid=str(uuid.uuid4()).split('-')[0]
    )

    command = ['attic', 'create']
    if stats:
        command.append('--stats')
    if verbosity > 1:
        command.append('--verbose')

    if 'exclude' in setup:
        for path in setup['exclude']:
            command.append('--exclude')
            command.append(expand(path))

    command.append(get_repo_archive_name(repository, archive_name))
    command.extend([expand(x) for x in setup['paths']])
    return command


def create_prune_command(repository, prefix, setup, stats=False, verbosity=0):
    command = ['attic', 'prune']
    if stats:
        command.append('--stats')
    if verbosity > 1:
        command.append('--verbose')

    for suffix in ['within', 'hourly', 'daily', 'weekly', 'monthly', 'yearly']:
        if suffix in setup:
            command.append('--keep-{0}'.format(suffix))
            command.append('{0}'.format(setup[suffix]))

    command.append('--prefix')
    command.append('{0}-'.format(prefix))
    command.append(repository)
    return command


def execute_attic_command(command, verbosity=0, dry_run=False, passphrase=None):
    if verbosity or dry_run:
        stdout_write('> Executing: "{0}"\n'.format(' '.join(command)))
        if dry_run:
            return

    env = None
    if passphrase:
        env = os.environ.copy()
        env['ATTIC_PASSPHRASE'] = str(passphrase)
    return subprocess.Popen(command, env=env).wait()


def run(config_file, stats, verbose, args, dry_run, just_listem,
        only_prune, only_create):
    config_file = expand(config_file)
    repositories, prune_rules = parse_config(config_file)

    for idx, repository, prefix, setup in iter_repositories(repositories):
        repository_full = expand(repository)
        prune = setup.pop('prune', None)
        auto = setup.pop('auto', True)
        passphrase = setup.pop('passphrase', None)
        archive = get_repo_archive_name(repository, prefix)
        sha = sha12(repository_full, prefix)
        if just_listem:
            if not idx:
                listemfmt = '{id:<20} {auto:<9} {prune:<5} {archive}\n'
                stdout_write(listemfmt.format(
                    id='ID',
                    repository='REPOSITORY',
                    auto='AUTOMATIC',
                    prune='PRUNE',
                    archive='ARCHIVE',
                ))
            stdout_write(listemfmt.format(
                repository=repository,
                id=sha,
                auto='yes' if auto else 'no',
                archive=archive,
                prune='yes' if prune else '',
            ))
            continue

        if not args and not auto:
            if verbose:
                stdout_write('Skipping {archive}\n'.format(archive=archive))
            continue
        elif args:
            full_repo = get_repo_archive_name(repository_full, prefix)
            if full_repo not in args and sha not in args:
                continue

        if not only_prune:
            if verbose:
                stdout_write('Running {archive}\n'.format(archive=archive))

            command = create_create_command(
                repository=repository_full,
                prefix=prefix,
                setup=setup,
                stats=stats,
                verbosity=verbose,
            )
            if command:
                execute_attic_command(
                    command=command,
                    verbosity=verbose,
                    dry_run=dry_run,
                    passphrase=passphrase,
                )
        if only_create or not prune:
            continue
        prune = prune_rules.get(prune) if prune else None
        if prune:
            command = create_prune_command(
                repository=repository_full,
                prefix=prefix,
                setup=prune,
                stats=stats,
                verbosity=verbose,

            )
            if command:
                execute_attic_command(
                    command=command,
                    verbosity=verbose,
                    dry_run=dry_run,
                    passphrase=passphrase,
                )
